@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: downloader-view.c,v 1.19 2003/06/28 23:44:08 mpeseng Exp $
+ *  $Id: downloader-view.c,v 1.24.2.1 2003/12/06 13:57:16 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -325,6 +325,7 @@ controls_info_foreach (GtkTreeModel *model,
 
 	gtk_tree_model_get_value (model, iter, COL_PERSIST_OBJECT, &val);
 	persist_object = g_value_get_pointer (&val);
+	g_value_unset (&val);
 
 	details = g_hash_table_lookup (info->priv->details_hash,
 				       persist_object);
@@ -393,19 +394,19 @@ downloader_view_update_details (DownloaderViewPrivate *priv,
 	}
 	else if (details->size_total > 0)
 	{
-		sprintf (buffer, _("%d of %d KB"),
+		sprintf (buffer, _("%d of %d kB"),
 			 details->size_done,
 			 details->size_total);
 	}
 	else
 	{
-		sprintf (buffer, _("%d KB"),
+		sprintf (buffer, _("%d kB"),
 			 details->size_done);
 	}
 
 	if (details->speed > 0)
 	{
-		sprintf (buffer, _("%s at %.1f KB/s"), buffer, details->speed);
+		sprintf (buffer, _("%s at %.1f kB/s"), buffer, details->speed);
 	}
 	gtk_label_set_text (GTK_LABEL (priv->details_status),
 			    buffer);
@@ -459,7 +460,7 @@ downloader_view_set_download_info (DownloaderViewPrivate *priv,
 				   GtkTreeIter *iter)
 {
 	gchar buffer[50];
-	GtkTreePath *path;
+	GtkTreePath *path = NULL;
 	GtkTreePath *selected_path = NULL;
 	GtkTreeIter selected_iter;
 	GtkTreeSelection *selection;
@@ -516,7 +517,7 @@ downloader_view_set_download_info (DownloaderViewPrivate *priv,
 	}
 	else if (details->size_total > 0)
 	{
-		sprintf (buffer, "%d KB", details->size_total);
+		sprintf (buffer, "%d kB", details->size_total);
 	}
 	else
 	{
@@ -550,6 +551,9 @@ downloader_view_set_download_info (DownloaderViewPrivate *priv,
 	{
 		downloader_view_update_details (priv, details);
 	}
+
+	gtk_tree_path_free (path);
+	gtk_tree_path_free (selected_path);
 }
 
 static void
@@ -584,6 +588,7 @@ downloader_view_add_download (DownloaderView *dv,
 	GtkTreeIter iter;
 	DownloadDetails *details;
 	GtkTreeSelection *selection;
+	GtkTreePath *path;
 
 	details = g_new0 (DownloadDetails, 1);
 	details->filename = g_strdup (filename);
@@ -605,10 +610,10 @@ downloader_view_add_download (DownloaderView *dv,
 	gtk_list_store_append (GTK_LIST_STORE (dv->priv->model),
 			       &iter);
 
+	path =  gtk_tree_model_get_path (GTK_TREE_MODEL (dv->priv->model), &iter);
 	details->ref = gtk_tree_row_reference_new
-		(GTK_TREE_MODEL (dv->priv->model),
-		 gtk_tree_model_get_path
-		 (GTK_TREE_MODEL (dv->priv->model), &iter));
+				(GTK_TREE_MODEL (dv->priv->model),path);
+	gtk_tree_path_free (path);
 
 	gtk_list_store_set (GTK_LIST_STORE (dv->priv->model),
 			    &iter,
@@ -630,15 +635,17 @@ downloader_view_remove_download (DownloaderView *dv,
 				 gpointer persist_object)
 {
 	DownloadDetails *details;
+	GtkTreePath *path = NULL;
 	GtkTreeIter iter;
 
 	details = g_hash_table_lookup (dv->priv->details_hash,
 				       persist_object);
 	g_return_if_fail (details);
 
+	path = gtk_tree_row_reference_get_path (details->ref);
+
 	gtk_tree_model_get_iter (GTK_TREE_MODEL (dv->priv->model),
-				 &iter,
-				 gtk_tree_row_reference_get_path (details->ref));
+				 &iter, path);
 
 	gtk_list_store_remove (GTK_LIST_STORE (dv->priv->model), &iter);
 
@@ -646,6 +653,8 @@ downloader_view_remove_download (DownloaderView *dv,
 			     persist_object);
 
 	ensure_selected_row (dv);
+
+	gtk_tree_path_free (path);
 }
 
 void
@@ -659,6 +668,7 @@ downloader_view_set_download_progress (DownloaderView *dv,
 				       gpointer persist_object)
 {
 	DownloadDetails *details;
+	GtkTreePath *path = NULL;
 	GtkTreeIter iter;
 
 	details = g_hash_table_lookup (dv->priv->details_hash,
@@ -672,11 +682,14 @@ downloader_view_set_download_progress (DownloaderView *dv,
 	details->size_done = size_done;
 	details->progress = progress;
 
+	path = gtk_tree_row_reference_get_path (details->ref);
+
 	gtk_tree_model_get_iter (GTK_TREE_MODEL (dv->priv->model),
-				 &iter,
-				 gtk_tree_row_reference_get_path (details->ref));
+				 &iter, path);
 
 	downloader_view_set_download_info (dv->priv, details, &iter);
+
+	gtk_tree_path_free (path);
 }
 
 void
@@ -685,6 +698,7 @@ downloader_view_set_download_status (DownloaderView *dv,
 				     gpointer persist_object)
 {
 	DownloadDetails *details;
+	GtkTreePath *path = NULL;
 	GtkTreeIter iter;
 
 	details = g_hash_table_lookup (dv->priv->details_hash,
@@ -693,9 +707,10 @@ downloader_view_set_download_status (DownloaderView *dv,
 
 	details->status = status;
 
+	path = gtk_tree_row_reference_get_path (details->ref);
+
 	gtk_tree_model_get_iter (GTK_TREE_MODEL (dv->priv->model),
-				 &iter,
-				 gtk_tree_row_reference_get_path (details->ref));
+				 &iter, path);
 
 	downloader_view_set_download_info (dv->priv, details, &iter);
 	downloader_view_update_controls (dv->priv);
@@ -704,6 +719,8 @@ downloader_view_set_download_status (DownloaderView *dv,
 	{
 		downloader_view_remove_download (dv, persist_object);
 	}*/
+
+	gtk_tree_path_free (path);
 }
 
 static void
@@ -832,6 +849,7 @@ download_dialog_pause_cb (GtkButton *button, DownloaderView *dv)
 
 	gtk_tree_model_get_value (model, &iter, COL_PERSIST_OBJECT, &val);
 	persist_object = g_value_get_pointer (&val);
+	g_value_unset (&val);
 
 	details = g_hash_table_lookup (dv->priv->details_hash,
 				       persist_object);
@@ -872,17 +890,20 @@ download_dialog_abort_cb (GtkButton *button, DownloaderView *dv)
         for (r = rlist; r != NULL; r = r->next)
         {
                 GtkTreeRowReference *node = r->data;
+		GtkTreePath *path = NULL;
 		GValue val = {0, };
 		gpointer *persist_object;
 		GtkTreeIter iter;
 		DownloadDetails *details;
 
-                gtk_tree_model_get_iter (model, &iter,
-                                         gtk_tree_row_reference_get_path (node));
+		path = gtk_tree_row_reference_get_path (node);
+
+                gtk_tree_model_get_iter (model, &iter, path);
 
 		gtk_tree_model_get_value (model, &iter,
 					  COL_PERSIST_OBJECT, &val);
 		persist_object = g_value_get_pointer (&val);
+		g_value_unset (&val);
 
 		details = g_hash_table_lookup (dv->priv->details_hash,
 					       persist_object);
@@ -893,6 +914,7 @@ download_dialog_abort_cb (GtkButton *button, DownloaderView *dv)
 		downloader_view_remove_download (dv, persist_object);
 
                 gtk_tree_row_reference_free (node);
+		gtk_tree_path_free (path);
         }
 
 	g_list_foreach (llist, (GFunc)gtk_tree_path_free, NULL);
@@ -921,6 +943,7 @@ downloader_treeview_selection_changed_cb (GtkTreeSelection *selection,
 	{
 		gtk_tree_model_get_value (priv->model, &iter, COL_PERSIST_OBJECT, &val);
 		persist_object = g_value_get_pointer (&val);
+		g_value_unset (&val);
 
 		details = g_hash_table_lookup (priv->details_hash,
 					       persist_object);
@@ -964,8 +987,11 @@ delete_pending_foreach  (gpointer persist_object,
 			 DownloadDetails *details,
 			 DownloaderView *dv)
 {
-	g_signal_emit (G_OBJECT (dv), downloader_view_signals[DOWNLOAD_REMOVE],
-		       0, persist_object);
+	if (details->status != DOWNLOAD_STATUS_COMPLETED)
+        {
+		g_signal_emit (G_OBJECT (dv), downloader_view_signals[DOWNLOAD_REMOVE],
+			       0, persist_object);
+	}
 
 	return TRUE;
 }
@@ -1041,6 +1067,7 @@ open_selection_foreach (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *ite
 
 	gtk_tree_model_get_value (model, iter, COL_PERSIST_OBJECT, &val);
 	persist_object = g_value_get_pointer (&val);
+	g_value_unset (&val);
 
 	details = g_hash_table_lookup (dv->priv->details_hash,
 				       persist_object);
@@ -1049,7 +1076,7 @@ open_selection_foreach (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *ite
 	if (details->status != DOWNLOAD_STATUS_COMPLETED) return;
 
 	mime = gnome_vfs_get_mime_type (details->dest);
-	g_return_if_fail (mime != NULL);
+	if (mime == NULL) return;
 
 	app = gnome_vfs_mime_get_default_application (mime);
 	if (app)
@@ -1057,6 +1084,8 @@ open_selection_foreach (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *ite
 		ephy_file_launch_application (app->command,
 					      details->dest,
 					      app->requires_terminal);
+
+		gnome_vfs_mime_application_free(app);
 	}
 	else
 	{
