@@ -14,10 +14,12 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: ephy-history.c,v 1.27.2.3 2003/12/18 12:48:42 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include "ephy-types.h"
@@ -47,10 +49,9 @@ struct EphyHistoryPrivate
 	EphyNode *pages;
 	EphyNode *last_page;
 	GHashTable *hosts_hash;
-	GStaticRWLock *hosts_hash_lock;
 	GHashTable *pages_hash;
-	GStaticRWLock *pages_hash_lock;
 	int autosave_timeout;
+	guint update_hosts_idle;
 };
 
 enum
@@ -159,7 +160,6 @@ ephy_history_autocompletion_source_foreach (EphyAutocompletionSource *source,
 		      url, url, FALSE,
 		      FALSE, score, data);
 	}
-	ephy_node_thaw (eb->priv->pages);
 }
 
 static void
@@ -206,7 +206,11 @@ ephy_history_load (EphyHistory *eb)
 		return;
 
 	doc = xmlParseFile (eb->priv->xml_file);
-	g_return_if_fail (doc != NULL);
+	if (doc == NULL)
+	{
+		g_warning ("Failed to load history from %s!\n", eb->priv->xml_file);
+		return;
+	}
 
 	root = xmlDocGetRootElement (doc);
 
@@ -249,7 +253,6 @@ remove_obsolete_pages (EphyHistory *eb)
         g_date_set_time (&current_date, time (NULL));
 
 	children = ephy_node_get_children (eb->priv->pages);
-	ephy_node_thaw (eb->priv->pages);
 	for (i = 0; i < children->len; i++)
 	{
 		EphyNode *kid;
@@ -291,7 +294,6 @@ ephy_history_save (EphyHistory *eb)
 
 		ephy_node_save_to_xml (kid, root);
 	}
-	ephy_node_thaw (eb->priv->hosts);
 
 	children = ephy_node_get_children (eb->priv->pages);
 	for (i = 0; i < children->len; i++)
@@ -302,7 +304,6 @@ ephy_history_save (EphyHistory *eb)
 
 		ephy_node_save_to_xml (kid, root);
 	}
-	ephy_node_thaw (eb->priv->pages);
 
 	ephy_file_save_xml (eb->priv->xml_file, doc);
 	xmlFreeDoc(doc);
@@ -313,13 +314,9 @@ hosts_added_cb (EphyNode *node,
 	        EphyNode *child,
 	        EphyHistory *eb)
 {
-	g_static_rw_lock_writer_lock (eb->priv->hosts_hash_lock);
-
 	g_hash_table_insert (eb->priv->hosts_hash,
 			     (char *) ephy_node_get_property_string (child, EPHY_NODE_PAGE_PROP_LOCATION),
 			     child);
-
-	g_static_rw_lock_writer_unlock (eb->priv->hosts_hash_lock);
 }
 
 static void
@@ -328,12 +325,8 @@ hosts_removed_cb (EphyNode *node,
 		  guint old_index,
 		  EphyHistory *eb)
 {
-	g_static_rw_lock_writer_lock (eb->priv->hosts_hash_lock);
-
 	g_hash_table_remove (eb->priv->hosts_hash,
 			     ephy_node_get_property_string (child, EPHY_NODE_PAGE_PROP_LOCATION));
-
-	g_static_rw_lock_writer_unlock (eb->priv->hosts_hash_lock);
 }
 
 static void
@@ -341,13 +334,9 @@ pages_added_cb (EphyNode *node,
 	        EphyNode *child,
 	        EphyHistory *eb)
 {
-	g_static_rw_lock_writer_lock (eb->priv->pages_hash_lock);
-
 	g_hash_table_insert (eb->priv->pages_hash,
 			     (char *) ephy_node_get_property_string (child, EPHY_NODE_PAGE_PROP_LOCATION),
 			     child);
-
-	g_static_rw_lock_writer_unlock (eb->priv->pages_hash_lock);
 }
 
 static void
@@ -356,12 +345,8 @@ pages_removed_cb (EphyNode *node,
 		  guint old_index,
 		  EphyHistory *eb)
 {
-	g_static_rw_lock_writer_lock (eb->priv->pages_hash_lock);
-
 	g_hash_table_remove (eb->priv->pages_hash,
 			     ephy_node_get_property_string (child, EPHY_NODE_PAGE_PROP_LOCATION));
-
-	g_static_rw_lock_writer_unlock (eb->priv->pages_hash_lock);
 }
 
 static gboolean
@@ -373,10 +358,75 @@ periodic_save_cb (EphyHistory *eh)
 	return TRUE;
 }
 
-static gboolean
-unref_empty_host (EphyNode *node)
+static void
+update_host_on_child_remove (EphyNode *node)
 {
-	ephy_node_unref (node);
+	GPtrArray *children;
+	int i, host_last_visit, new_host_last_visit = 0;
+
+	host_last_visit = ephy_node_get_property_int
+			(node, EPHY_NODE_PAGE_PROP_LAST_VISIT);
+
+	children = ephy_node_get_children (node);
+	for (i = 0; i < children->len; i++)
+	{
+		EphyNode *kid;
+		int last_visit;
+
+		kid = g_ptr_array_index (children, i);
+
+		last_visit = ephy_node_get_property_int
+                        (kid, EPHY_NODE_PAGE_PROP_LAST_VISIT);
+
+		if (last_visit > new_host_last_visit)
+		{
+			new_host_last_visit = last_visit;
+		}
+	}
+
+	if (host_last_visit != new_host_last_visit)
+	{
+		GValue value = { 0, };
+
+		g_value_init (&value, G_TYPE_INT);
+		g_value_set_int (&value, new_host_last_visit);
+		ephy_node_set_property (node, EPHY_NODE_PAGE_PROP_LAST_VISIT,
+				        &value);
+		g_value_unset (&value);
+	}
+}
+
+static gboolean
+update_hosts (EphyHistory *eh)
+{
+	GPtrArray *children;
+	int i;
+	GList *empty = NULL;
+
+	children = ephy_node_get_children (eh->priv->hosts);
+	for (i = 0; i < children->len; i++)
+	{
+		EphyNode *kid;
+
+		kid = g_ptr_array_index (children, i);
+
+		if (kid != eh->priv->pages)
+		{
+			if (ephy_node_get_n_children (kid) > 0)
+			{
+				update_host_on_child_remove (kid);
+			}
+			else
+			{
+				empty = g_list_prepend (empty, kid);
+			}
+		}
+	}
+
+	g_list_foreach (empty, (GFunc)ephy_node_unref, NULL);
+	g_list_free (empty);
+
+	eh->priv->update_hosts_idle = 0;
 
 	return FALSE;
 }
@@ -387,9 +437,10 @@ page_removed_from_host_cb (EphyNode *node,
 		           guint old_index,
 		           EphyHistory *eb)
 {
-	if (ephy_node_get_n_children (node) == 0)
+	if (!eb->priv->update_hosts_idle)
 	{
-		g_idle_add ((GSourceFunc)unref_empty_host, node);
+		eb->priv->update_hosts_idle = g_idle_add
+			((GSourceFunc)update_hosts, eb);
 	}
 }
 
@@ -414,6 +465,8 @@ ephy_history_init (EphyHistory *eb)
 
         eb->priv = g_new0 (EphyHistoryPrivate, 1);
 
+	eb->priv->update_hosts_idle = 0;
+
 	db = ephy_node_db_new (EPHY_NODE_DB_HISTORY);
 	eb->priv->db = db;
 
@@ -423,13 +476,8 @@ ephy_history_init (EphyHistory *eb)
 
 	eb->priv->pages_hash = g_hash_table_new (g_str_hash,
 			                          g_str_equal);
-	eb->priv->pages_hash_lock = g_new0 (GStaticRWLock, 1);
-	g_static_rw_lock_init (eb->priv->pages_hash_lock);
-
 	eb->priv->hosts_hash = g_hash_table_new (g_str_hash,
 			                         g_str_equal);
-	eb->priv->hosts_hash_lock = g_new0 (GStaticRWLock, 1);
-	g_static_rw_lock_init (eb->priv->hosts_hash_lock);
 
 	/* Pages */
 	eb->priv->pages = ephy_node_new_with_id (db, PAGES_NODE_ID);
@@ -497,6 +545,11 @@ ephy_history_finalize (GObject *object)
 
         g_return_if_fail (eb->priv != NULL);
 
+	if (eb->priv->update_hosts_idle)
+	{
+		g_source_remove (eb->priv->update_hosts_idle);
+	}
+
 	ephy_history_save (eb);
 
 	ephy_node_unref (eb->priv->pages);
@@ -505,9 +558,7 @@ ephy_history_finalize (GObject *object)
 	g_object_unref (eb->priv->db);
 
 	g_hash_table_destroy (eb->priv->pages_hash);
-	g_static_rw_lock_free (eb->priv->pages_hash_lock);
 	g_hash_table_destroy (eb->priv->hosts_hash);
-	g_static_rw_lock_free (eb->priv->hosts_hash_lock);
 
 	g_source_remove (eb->priv->autosave_timeout);
 
@@ -588,7 +639,7 @@ ephy_history_add_host (EphyHistory *eh, EphyNode *page)
 		host_locations = g_list_append (host_locations,
 						g_strdup ("about:blank"));
 	}
-	else if (strcmp (url, "file") == 0)
+	else if (strcmp (scheme, "file") == 0)
 	{
 		host_name = _("Local files");
 		host_locations = g_list_append (host_locations,
@@ -599,11 +650,10 @@ ephy_history_add_host (EphyHistory *eh, EphyNode *page)
 		char *location;
 		char *tmp;
 
-		location = g_strconcat (gnome_vfs_uri_get_scheme (vfs_uri),
+		location = g_strconcat (scheme,
 					"://", host_name, "/", NULL);
 		host_locations = g_list_append (host_locations, location);
 
-
 		if (g_str_has_prefix (host_name, "www."))
 		{
 			tmp = g_strdup (g_utf8_offset_to_pointer (host_name, 4));
@@ -620,15 +670,12 @@ ephy_history_add_host (EphyHistory *eh, EphyNode *page)
 
 	g_return_val_if_fail (host_locations != NULL, NULL);
 
-	g_static_rw_lock_reader_lock (eh->priv->hosts_hash_lock);
-
 	for (l = host_locations; l != NULL; l = l->next)
 	{
 		host = g_hash_table_lookup (eh->priv->hosts_hash,
 					    (char *)l->data);
 		if (host) break;
 	}
-	g_static_rw_lock_reader_unlock (eh->priv->hosts_hash_lock);
 
 	if (!host)
 	{
@@ -787,9 +834,7 @@ ephy_history_get_page (EphyHistory *eb,
 {
 	EphyNode *node;
 
-	g_static_rw_lock_reader_lock (eb->priv->pages_hash_lock);
 	node = g_hash_table_lookup (eb->priv->pages_hash, url);
-	g_static_rw_lock_reader_unlock (eb->priv->pages_hash_lock);
 
 	return node;
 }
@@ -823,6 +868,26 @@ ephy_history_set_page_title (EphyHistory *gh,
 	g_value_unset (&value);
 }
 
+const char*
+ephy_history_get_icon (EphyHistory *gh,
+		       const char *url)
+{
+	EphyNode *node, *host;
+	int host_id;
+
+	node = ephy_history_get_page (gh, url);
+	if (node == NULL) return NULL;
+	
+	host_id = ephy_node_get_property_int (node, EPHY_NODE_PAGE_PROP_HOST_ID);
+	g_return_val_if_fail (host_id >= 0, NULL);
+
+	host = ephy_node_db_get_node_from_id (gh->priv->db, host_id);
+	g_return_val_if_fail (host != NULL, NULL);
+
+	return ephy_node_get_property_string (host, EPHY_NODE_PAGE_PROP_ICON);
+}	
+	
+		       
 void
 ephy_history_set_icon (EphyHistory *gh,
 		       const char *url,
