@@ -14,10 +14,12 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: mozilla-embed-single.cpp,v 1.36.2.7 2003/12/29 20:28:18 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include "glib.h"
@@ -49,12 +51,14 @@
 #include <nsIFontEnumerator.h>
 #include <nsISupportsPrimitives.h>
 #include <nsReadableUtils.h>
-#include <nsICookieManager.h>
 #include <nsIPasswordManager.h>
 #include <nsIPassword.h>
 #include <nsICookie.h>
-#include <nsCCookieManager.h>
 #include <nsCPasswordManager.h>
+#include <nsICookieManager.h>
+#if MOZILLA_SNAPSHOT < 13
+#include <nsCCookieManager.h>
+#endif
 
 // FIXME: For setting the locale. hopefully gtkmozembed will do itself soon
 #include <nsIChromeRegistry.h>
@@ -553,9 +557,14 @@ getUILang (nsAString& aUILang)
 		return NS_ERROR_FAILURE;
 	}
 
+#if MOZILLA_SNAPSHOT >= 12
+	result = localeService->GetLocaleComponentForUserAgent (aUILang);
+#else
 	nsXPIDLString uiLang;
 	result = localeService->GetLocaleComponentForUserAgent (getter_Copies(uiLang));
 	aUILang = uiLang;
+#endif
+
 	if (NS_FAILED (result))
 	{
 		g_warning ("Could not determine locale!\n");
@@ -888,7 +897,7 @@ impl_list_cookies (EphyEmbedSingle *shell,
         nsresult result;
 
         nsCOMPtr<nsICookieManager> cookieManager = 
-                        do_CreateInstance (NS_COOKIEMANAGER_CONTRACTID);
+                        do_GetService (NS_COOKIEMANAGER_CONTRACTID);
         nsCOMPtr<nsISimpleEnumerator> cookieEnumerator;
         result = 
             cookieManager->GetEnumerator (getter_AddRefs(cookieEnumerator));
@@ -947,7 +956,7 @@ impl_remove_cookies (EphyEmbedSingle *shell,
 	nsresult result;
 	GList *cl;
         nsCOMPtr<nsICookieManager> cookieManager =
-                        do_CreateInstance (NS_COOKIEMANAGER_CONTRACTID);
+                        do_GetService (NS_COOKIEMANAGER_CONTRACTID);
 	
         for (cl = g_list_first(cookies) ; cl != NULL ; 
              cl = g_list_next (cl))
@@ -972,7 +981,7 @@ impl_list_passwords (EphyEmbedSingle *shell,
         nsresult result = NS_ERROR_FAILURE;
 
         nsCOMPtr<nsIPasswordManager> passwordManager =
-                        do_CreateInstance (NS_PASSWORDMANAGER_CONTRACTID);
+                        do_GetService (NS_PASSWORDMANAGER_CONTRACTID);
         nsCOMPtr<nsISimpleEnumerator> passwordEnumerator;
         if (type == PASSWORD_PASSWORD)
                 result = passwordManager->GetEnumerator 
@@ -1020,7 +1029,7 @@ impl_remove_passwords (EphyEmbedSingle *shell,
 {
 	nsresult result = NS_ERROR_FAILURE;
         nsCOMPtr<nsIPasswordManager> passwordManager =
-                        do_CreateInstance (NS_PASSWORDMANAGER_CONTRACTID);
+                        do_GetService (NS_PASSWORDMANAGER_CONTRACTID);
 
         for (GList *l = g_list_first(passwords) ; l !=NULL ; 
              l = g_list_next(l))
