@@ -145,10 +145,6 @@ impl_get_security_level (EphyEmbed *embed,
                          EmbedSecurityLevel *level,
                          char **description);
 static gresult
-impl_find (EphyEmbed *embed,
-           EmbedFindInfo *info);
-
-static gresult
 impl_set_encoding (EphyEmbed *embed,
                    const char *encoding);
 
@@ -295,6 +291,52 @@ mozilla_embed_get_type (void)
         return mozilla_embed_type;
 }
 
+static gresult
+impl_find_next (EphyEmbed *embed, 
+                gboolean backwards)
+{
+	nsresult result = NS_OK;
+	EphyWrapper *wrapper;
+	
+	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
+	g_return_val_if_fail (wrapper != NULL, G_FAILED);
+	
+        PRBool didFind;
+
+        result = wrapper->Find (backwards, &didFind);
+	
+	return didFind ? G_OK : G_FAILED;
+}
+
+static gresult
+impl_activate (EphyEmbed *embed) 
+{
+	g_return_val_if_fail (IS_EPHY_EMBED (embed), G_FAILED);
+
+	gtk_widget_grab_focus (GTK_BIN (embed)->child);
+	
+	return G_OK;
+}
+
+static gresult
+impl_find_set_properties (EphyEmbed *embed, 
+                          char *search_string,
+	                  gboolean case_sensitive,
+			  gboolean wrap_around)
+{
+	nsresult result = NS_OK;
+	EphyWrapper *wrapper;
+	
+	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
+	g_return_val_if_fail (wrapper != NULL, G_FAILED);
+	
+        result = wrapper->FindSetProperties
+		((NS_ConvertUTF8toUCS2(search_string)).get(),
+		 case_sensitive, wrap_around); 
+	
+	return result ? G_OK : G_FAILED;
+}
+
 static void
 ephy_embed_init (EphyEmbedClass *embed_class)
 {
@@ -330,7 +372,9 @@ ephy_embed_init (EphyEmbedClass *embed_class)
 	embed_class->shistory_go_nth = impl_shistory_go_nth;
 	embed_class->shistory_copy = impl_shistory_copy;
 	embed_class->get_security_level = impl_get_security_level;
-	embed_class->find = impl_find;
+	embed_class->find_next = impl_find_next;
+	embed_class->activate = impl_activate;
+	embed_class->find_set_properties = impl_find_set_properties;
 	embed_class->set_encoding = impl_set_encoding;
 	embed_class->select_all = impl_select_all;
 	embed_class->print = impl_print;
@@ -354,6 +398,7 @@ static void
 mozilla_embed_init (MozillaEmbed *embed)
 {
         embed->priv = g_new0 (MozillaEmbedPrivate, 1);
+	embed->priv->no_page = 1;
 
 	mozilla_embed_connect_signals (embed);
 }
@@ -495,22 +540,24 @@ impl_can_go_forward (EphyEmbed *embed)
 static gresult 
 impl_can_go_up (EphyEmbed *embed)
 {
-	char *location;
+	char *location = NULL;
 	char *s;
-	gresult result;
+	gresult result = G_FAILED;
 
 	if (ephy_embed_get_location (embed, TRUE, &location) != G_OK)
-	  return G_FAILED;
+	{
+		g_free (location);
+
+		return G_FAILED;
+	}
+
 	g_return_val_if_fail (location != NULL, G_FAILED);
+
 	if ((s = mozilla_embed_get_uri_parent (location)) != NULL)
 	{
 		g_free (s);
 		result = G_OK;
 	}
-	else
-	{
-		result = G_FAILED;
-	}
 
 	g_free (location);
 
@@ -520,14 +567,20 @@ impl_can_go_up (EphyEmbed *embed)
 static gresult
 impl_get_go_up_list (EphyEmbed *embed, GSList **l)
 {
-	char *location;
+	char *location = NULL;
 	char *s;
 	
+	*l = NULL;
+
 	if (ephy_embed_get_location (embed, TRUE, &location) != G_OK)
+	{
+		g_free (location);
+
 		return G_FAILED;
+	}
+
 	g_return_val_if_fail (location != NULL, G_FAILED);
 	
-	*l = NULL;
 	s = location;
 	while ((s = mozilla_embed_get_uri_parent (s)) != NULL)
 	{
@@ -535,6 +588,7 @@ impl_get_go_up_list (EphyEmbed *embed, GSList **l)
 	}				
 
 	g_free (location);
+
 	*l = g_slist_reverse (*l);
 
 	return G_OK;
@@ -559,15 +613,25 @@ impl_go_forward (EphyEmbed *embed)
 static gresult
 impl_go_up (EphyEmbed *embed)
 {
-	char *uri;
+	char *uri = NULL;
 	char *parent_uri;
-	
-	ephy_embed_get_location (embed, TRUE, &uri);
+
+	if (ephy_embed_get_location (embed, TRUE, &uri) != G_OK)
+	{
+		g_free (uri);
+
+		return G_FAILED;
+	}
+
 	g_return_val_if_fail (uri != NULL, G_FAILED);
 	
 	parent_uri = mozilla_embed_get_uri_parent (uri);
 	g_free (uri);
-	g_return_val_if_fail (parent_uri != NULL, G_FAILED);
+
+	if (parent_uri == NULL)
+	{
+		return G_FAILED;
+	}
 	
 	ephy_embed_load_url (embed, parent_uri);
 
@@ -698,14 +762,15 @@ impl_get_location (EphyEmbed *embed,
 	 * in fact the wrapper is initialized on net start */
 	if (!wrapper) 
 	{
-		*location = NULL;
+		*location = g_strdup ("about:blank");
 		return G_FAILED;
 	}
 	
 	if (toplevel)
 	{
-		l = gtk_moz_embed_get_location 
-			(GTK_MOZ_EMBED(embed));
+		rv = wrapper->GetMainDocumentUrl (url);
+		l = (NS_SUCCEEDED (rv) && !url.IsEmpty()) ?
+		     g_strdup (url.get()) : NULL;	   	
 	}
 	else
 	{
@@ -1092,6 +1157,8 @@ impl_print (EphyEmbed *embed,
 
 	result = wrapper->Print(options, info->preview);
 
+	options->SetPrintSilent (PR_FALSE);
+
 	return NS_SUCCEEDED (result) ? G_OK : G_FAILED;
 }
 
@@ -1137,28 +1204,6 @@ impl_print_preview_navigate (EphyEmbed *embed,
 	return NS_SUCCEEDED(result) ? G_OK : G_FAILED;
 }
 
-static gresult
-impl_find (EphyEmbed *embed, 
-           EmbedFindInfo *info)
-{
-	nsresult result = NS_OK;
-	EphyWrapper *wrapper;
-	
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
-	
-        PRBool didFind;
-
-        result = wrapper->Find ((NS_ConvertUTF8toUCS2(info->search_string)).get(),
-				info->interactive,
-				info->match_case, 
-                 	        info->backwards, info->wrap,
-                       	        info->entire_word, info->search_frames,
-                                &didFind);
-	
-	return didFind ? G_OK : G_FAILED;
-}
-
 static gresult
 impl_set_encoding (EphyEmbed *embed,
 		   const char *encoding)
@@ -1187,7 +1232,11 @@ mozilla_embed_location_changed_cb (GtkMozEmbed *embed,
 	 * to know about it. */
 	if (membed->priv->no_page != 0)
 	{
-		g_signal_emit_by_name (membed, "ge_location");
+		char *location;
+
+		location = gtk_moz_embed_get_location (embed);
+		g_signal_emit_by_name (membed, "ge_location", location);
+		g_free (location);
 	}
 
 	membed->priv->no_page = -1;
@@ -1320,6 +1369,12 @@ static gint
 mozilla_embed_dom_key_down_cb (GtkMozEmbed *embed, gpointer dom_event,
 		               MozillaEmbed *membed)
 {
+	if (dom_event == NULL)
+	{
+		g_warning ("mozilla_embed_dom_key_down_cb: domevent NULL");
+		return FALSE;
+	}
+
 	nsCOMPtr<nsIDOMKeyEvent> ev = static_cast<nsIDOMKeyEvent*>(dom_event);
 	if (!ev)
 	{
@@ -1338,9 +1393,11 @@ mozilla_embed_dom_key_down_cb (GtkMozEmbed *embed, gpointer dom_event,
 	EventContext ctx;
 	ctx.Init (wrapper);
 	rv = ctx.GetKeyEventInfo (ev, info);
-	if (NS_SUCCEEDED(rv) &&
-	    (info->keycode == nsIDOMKeyEvent::DOM_VK_F10 &&
-	     info->modifier == GDK_SHIFT_MASK))
+	if (NS_FAILED (rv)) return G_FAILED;
+
+	if (info->keycode == nsIDOMKeyEvent::DOM_VK_F10 &&
+	    (info->modifier == GDK_SHIFT_MASK ||
+	     info->modifier == GDK_CONTROL_MASK))
 	{
 		/* Translate relative coordinates to absolute values, and try
 		   to avoid covering links by adding a little offset. */
@@ -1350,6 +1407,11 @@ mozilla_embed_dom_key_down_cb (GtkMozEmbed *embed, gpointer dom_event,
 		info->x += x + 6;	
 		info->y += y + 6;
 
+		if (info->modifier == GDK_CONTROL_MASK)
+		{
+			info->context = EMBED_CONTEXT_DOCUMENT;	
+		}
+
 		nsCOMPtr<nsIDOMDocument> doc;
 		rv = ctx.GetTargetDocument (getter_AddRefs(doc));
 		if (NS_SUCCEEDED(rv))
@@ -1384,6 +1446,12 @@ mozilla_embed_dom_mouse_click_cb (GtkMozEmbed *embed, gpointer dom_event,
 	EphyWrapper *wrapper;
 	nsresult result;
 
+	if (dom_event == NULL)
+	{
+		g_warning ("mozilla_embed_dom_mouse_click_cb: domevent NULL");
+		return FALSE;
+	}
+
 	info = ephy_embed_event_new ();
 	
 	wrapper = MOZILLA_EMBED(membed)->priv->wrapper;
@@ -1420,36 +1488,48 @@ mozilla_embed_dom_mouse_down_cb (GtkMozEmbed *embed, gpointer dom_event,
 {
 	EphyEmbedEvent *info;
 	EventContext event_context;
-	gint return_value = 0;
+	gint return_value = FALSE;
 	EphyWrapper *wrapper;
 	nsresult result;
 	EphyEmbedEventType type;
 
+	if (dom_event == NULL)
+	{
+		g_warning ("mozilla_embed_dom_mouse_down_cb: domevent NULL");
+		return FALSE;
+	}
+
 	info = ephy_embed_event_new ();
 	
 	wrapper = MOZILLA_EMBED(membed)->priv->wrapper;
 	g_return_val_if_fail (wrapper != NULL, G_FAILED);
-	
+
 	event_context.Init (wrapper);
         result = event_context.GetMouseEventInfo (static_cast<nsIDOMMouseEvent*>(dom_event), info);
+	if (NS_FAILED (result)) return FALSE;
 
 	ephy_embed_event_get_event_type (info, &type);
-
-	if (NS_SUCCEEDED(result) && (type == EPHY_EMBED_EVENT_MOUSE_BUTTON3))
+		
+	nsCOMPtr<nsIDOMDocument> domDoc;
+	result = event_context.GetTargetDocument (getter_AddRefs(domDoc));
+	if (NS_SUCCEEDED(result))
 	{
-		nsCOMPtr<nsIDOMDocument> domDoc;
-		result = event_context.GetTargetDocument (getter_AddRefs(domDoc));
+		result = wrapper->PushTargetDocument (domDoc);
+
 		if (NS_SUCCEEDED(result))
 		{
-			result = wrapper->PushTargetDocument (domDoc);
-			if (NS_SUCCEEDED(result))
+			g_signal_emit_by_name (membed, "ge_dom_mouse_down", 
+					       info, &return_value); 
+
+			if (return_value == FALSE &&
+			    type == EPHY_EMBED_EVENT_MOUSE_BUTTON3)
 			{
 				g_signal_emit_by_name (membed, "ge_context_menu", 
-						       info, &return_value); 
-				wrapper->PopTargetDocument ();
+						       info, &return_value);
 			}
-		}
 
+			wrapper->PopTargetDocument ();
+		}
 	}
 
 	g_object_unref (info);
