@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-node-view.c,v 1.26.2.1 2003/09/04 22:19:43 marco Exp $
+ *  $Id: ephy-node-view.c,v 1.26.2.2 2003/12/18 12:45:37 chpe Exp $
  */
 
 #include <gtk/gtktreeselection.h>
@@ -616,12 +616,19 @@ ephy_node_view_select_node_by_key (EphyNodeView *view, GdkEventKey *event)
 	gchar outbuf[6];
 	gint length;
 
+	/* Work around bug 129411 */
+	if (!gtk_tree_model_get_iter_first (view->priv->filtermodel, &iter))
+	{
+		return FALSE;
+	}
+
 	length = g_unichar_to_utf8 (gdk_keyval_to_unicode (event->keyval), outbuf);
 	event_string = g_utf8_casefold (outbuf, length);
 
 	if (!gtk_tree_model_get_iter_first (view->priv->sortmodel, &iter))
 	{
 		g_free (event_string);
+		g_return_val_if_reached (FALSE);
 		return FALSE;
 	}
 
