@@ -119,28 +119,26 @@ sort_topics (gconstpointer a, gconstpointer b)
 {
 	EphyNode *node_a = (EphyNode *)a;
 	EphyNode *node_b = (EphyNode *)b;
-	EphyNodePriority priority_a, priority_b;
-	char *str_a = NULL;
-	char *str_b = NULL;
+	const char *title1, *title2;
 	int retval;
 
-	priority_a = ephy_node_get_property_int (node_a, EPHY_NODE_KEYWORD_PROP_PRIORITY);
-	priority_b = ephy_node_get_property_int (node_b, EPHY_NODE_KEYWORD_PROP_PRIORITY);
+	title1 = ephy_node_get_property_string (node_a, EPHY_NODE_KEYWORD_PROP_NAME);
+	title2 = ephy_node_get_property_string (node_b, EPHY_NODE_KEYWORD_PROP_NAME);
 
-	if (priority_a < priority_b)
+	if (title1 == NULL)
 	{
 		retval = -1;
 	}
-	else if (priority_a > priority_b)
+	else if (title2 == NULL)
 	{
 		retval = 1;
 	}
 	else
 	{
-		str_a = g_utf8_casefold (ephy_node_get_property_string (node_a, EPHY_NODE_KEYWORD_PROP_NAME),
-					 -1);
-		str_b = g_utf8_casefold (ephy_node_get_property_string (node_b, EPHY_NODE_KEYWORD_PROP_NAME),
-				         -1);
+		char *str_a, *str_b;
+
+		str_a = g_utf8_casefold (title1, -1);
+		str_b = g_utf8_casefold (title2, -1);
 		retval = g_utf8_collate (str_a, str_b);
 		g_free (str_a);
 		g_free (str_b);
@@ -154,17 +152,30 @@ sort_bookmarks (gconstpointer a, gconstpointer b)
 {
 	EphyNode *node_a = (EphyNode *)a;
 	EphyNode *node_b = (EphyNode *)b;
-	char *str_a = NULL;
-	char *str_b = NULL;
+	const char *title1, *title2;
 	int retval;
 
-	str_a = g_utf8_casefold (ephy_node_get_property_string (node_a, EPHY_NODE_BMK_PROP_TITLE),
-				 -1);
-	str_b = g_utf8_casefold (ephy_node_get_property_string (node_b, EPHY_NODE_BMK_PROP_TITLE),
-			         -1);
-	retval = g_utf8_collate (str_a, str_b);
-	g_free (str_a);
-	g_free (str_b);
+	title1 = ephy_node_get_property_string (node_a, EPHY_NODE_BMK_PROP_TITLE);
+	title2 = ephy_node_get_property_string (node_b, EPHY_NODE_BMK_PROP_TITLE);
+
+	if (title1 == NULL)
+	{
+		retval = -1;
+	}
+	else if (title2 == NULL)
+	{
+		retval = 1;
+	}
+	else
+	{
+		char *str_a, *str_b;
+
+		str_a = g_utf8_casefold (title1, -1);
+		str_b = g_utf8_casefold (title2, -1);
+		retval = g_utf8_collate (str_a, str_b);
+		g_free (str_a);
+		g_free (str_b);
+	}
 
 	return retval;
 }
@@ -228,8 +239,6 @@ add_bookmarks_menu (EphyBookmarksMenu *menu, EphyNode *node, GString *xml)
 
 		g_list_free (node_list);
 	}
-
-	ephy_node_thaw (node);
 }
 
 static void
@@ -287,7 +296,6 @@ ephy_bookmarks_menu_rebuild (EphyBookmarksMenu *menu)
 			node_list = g_list_prepend (node_list, kid);
 		}
 	}
-	ephy_node_thaw (topics);
 
 	node_list = g_list_sort (node_list, (GCompareFunc)sort_topics);
 
