@@ -235,16 +235,13 @@ fill_model (EphyTopicsSelector *editor)
 					    -1);
 		}
 	}
-	ephy_node_thaw (keywords);
 }
 
 static void
-topic_toggled (GtkCellRendererToggle *cell,
-               char *path_str,
+topic_toggled (GtkTreePath *path,
                EphyTopicsSelector *selector)
 {
 	GtkTreeModel *model = selector->priv->model;
-	GtkTreePath *path = gtk_tree_path_new_from_string (path_str);
 	GtkTreeIter iter;
 	gboolean has_topic;
 
@@ -307,7 +304,7 @@ topic_clicked (GtkTreeView *tree_view,
 	       GdkEventButton *event,
 	       EphyTopicsSelector *selector)
 {
-	GtkTreePath *path;
+	GtkTreePath *path = NULL;
 
 	if (event->window != gtk_tree_view_get_bin_window (tree_view))
 		return FALSE;
@@ -318,9 +315,9 @@ topic_clicked (GtkTreeView *tree_view,
 					   &path, NULL,
 					   NULL, NULL))
 	{
-		gchar *path_str = gtk_tree_path_to_string (path);
-		topic_toggled (NULL, path_str, selector);
-		g_free(path_str);
+		topic_toggled (path, selector);
+
+		gtk_tree_path_free (path);
 	}
 
 	return FALSE;
@@ -333,8 +330,7 @@ topic_key_pressed (GtkTreeView *tree_view,
 {
 	GtkTreeSelection *sel = NULL;
 	GtkTreeIter iter;
-	GtkTreePath *path = NULL;
-	gchar *path_str;
+	GtkTreePath *path;
 
 	switch (event->keyval)
 	{
@@ -346,9 +342,10 @@ topic_key_pressed (GtkTreeView *tree_view,
 		if (gtk_tree_selection_get_selected (sel, NULL, &iter))
 		{
 			path = gtk_tree_model_get_path (selector->priv->model, &iter);
-			path_str = gtk_tree_path_to_string (path);
-			topic_toggled (NULL, path_str, selector);
-			g_free(path_str);
+
+			topic_toggled (path, selector);
+
+			gtk_tree_path_free (path);
 		}
 		return TRUE;
 
